@@ -49,7 +49,7 @@ RCSID("$Id$");
 #define BITSIZE(TYPE)						\
 {								\
     int b = 0; TYPE x = 1, zero = 0; const char *pre = "u";	\
-    char tmp[128], tmp2[128];					\
+    char tmp[128], tmp2[192];					\
     while(x){ x <<= 1; b++; if(x < zero) pre=""; }		\
     if(b >= len){						\
         size_t tabs;						\
@@ -68,13 +68,13 @@ RCSID("$Id$");
 #endif
 
 static void
-try_signed(FILE *f, int len)  __attribute__ ((unused));
+try_signed(FILE *f, int len)  __attribute__ ((__unused__));
 
 static void
-try_unsigned(FILE *f, int len) __attribute__ ((unused));
+try_unsigned(FILE *f, int len) __attribute__ ((__unused__));
 
 static int
-print_bt(FILE *f, int flag) __attribute__ ((unused));
+print_bt(FILE *f, int flag) __attribute__ ((__unused__));
 
 static void
 try_signed(FILE *f, int len)
@@ -262,7 +262,7 @@ int main(int argc, char **argv)
 
     fprintf(f, "#ifndef HEIMDAL_DEPRECATED\n");
     fprintf(f, "#if __has_extension(deprecated) || KRB5TYPES_REQUIRE_GNUC(3,1,0)\n");
-    fprintf(f, "#define HEIMDAL_DEPRECATED __attribute__((deprecated))\n");
+    fprintf(f, "#define HEIMDAL_DEPRECATED __attribute__ ((__deprecated__))\n");
     fprintf(f, "#elif defined(_MSC_VER) && (_MSC_VER>1200)\n");
     fprintf(f, "#define HEIMDAL_DEPRECATED __declspec(deprecated)\n");
     fprintf(f, "#else\n");
@@ -272,7 +272,7 @@ int main(int argc, char **argv)
 
     fprintf(f, "#ifndef HEIMDAL_PRINTF_ATTRIBUTE\n");
     fprintf(f, "#if __has_extension(format) || KRB5TYPES_REQUIRE_GNUC(3,1,0)\n");
-    fprintf(f, "#define HEIMDAL_PRINTF_ATTRIBUTE(x) __attribute__((format x))\n");
+    fprintf(f, "#define HEIMDAL_PRINTF_ATTRIBUTE(x) __attribute__ ((__format__ x))\n");
     fprintf(f, "#else\n");
     fprintf(f, "#define HEIMDAL_PRINTF_ATTRIBUTE(x)\n");
     fprintf(f, "#endif\n");
@@ -280,7 +280,7 @@ int main(int argc, char **argv)
 
     fprintf(f, "#ifndef HEIMDAL_NORETURN_ATTRIBUTE\n");
     fprintf(f, "#if __has_extension(noreturn) || KRB5TYPES_REQUIRE_GNUC(3,1,0)\n");
-    fprintf(f, "#define HEIMDAL_NORETURN_ATTRIBUTE __attribute__((noreturn))\n");
+    fprintf(f, "#define HEIMDAL_NORETURN_ATTRIBUTE __attribute__ ((__noreturn__))\n");
     fprintf(f, "#else\n");
     fprintf(f, "#define HEIMDAL_NORETURN_ATTRIBUTE\n");
     fprintf(f, "#endif\n");
@@ -288,7 +288,7 @@ int main(int argc, char **argv)
 
     fprintf(f, "#ifndef HEIMDAL_UNUSED_ATTRIBUTE\n");
     fprintf(f, "#if __has_extension(unused) || KRB5TYPES_REQUIRE_GNUC(3,1,0)\n");
-    fprintf(f, "#define HEIMDAL_UNUSED_ATTRIBUTE __attribute__((unused))\n");
+    fprintf(f, "#define HEIMDAL_UNUSED_ATTRIBUTE __attribute__ ((__unused__))\n");
     fprintf(f, "#else\n");
     fprintf(f, "#define HEIMDAL_UNUSED_ATTRIBUTE\n");
     fprintf(f, "#endif\n");
@@ -296,7 +296,7 @@ int main(int argc, char **argv)
 
     fprintf(f, "#ifndef HEIMDAL_WARN_UNUSED_RESULT_ATTRIBUTE\n");
     fprintf(f, "#if __has_extension(warn_unused_result) || KRB5TYPES_REQUIRE_GNUC(3,3,0)\n");
-    fprintf(f, "#define HEIMDAL_WARN_UNUSED_RESULT_ATTRIBUTE __attribute__((warn_unused_result))\n");
+    fprintf(f, "#define HEIMDAL_WARN_UNUSED_RESULT_ATTRIBUTE __attribute__ ((__warn_unused_result__))\n");
     fprintf(f, "#else\n");
     fprintf(f, "#define HEIMDAL_WARN_UNUSED_RESULT_ATTRIBUTE\n");
     fprintf(f, "#endif\n");
