@@ -42,7 +42,8 @@ _gk_verify_buffers (
 	const gsskrb5_ctx /*ctx*/,
 	const gss_iov_buffer_desc */*header*/,
 	const gss_iov_buffer_desc */*padding*/,
-	const gss_iov_buffer_desc */*trailer*/);
+	const gss_iov_buffer_desc */*trailer*/,
+	int /*block_cipher*/);
 
 OM_uint32 GSSAPI_CALLCONV
 _gk_wrap_iov (
@@ -178,6 +179,17 @@ _gssapi_unwrap_cfx_iov (
 	gss_iov_buffer_desc */*iov*/,
 	int /*iov_count*/);
 
+OM_uint32
+_gssapi_unwrap_iov_arcfour (
+	OM_uint32 */*minor_status*/,
+	gsskrb5_ctx /*ctx*/,
+	krb5_context /*context*/,
+	int */*pconf_state*/,
+	gss_qop_t */*pqop_state*/,
+	gss_iov_buffer_desc */*iov*/,
+	int /*iov_count*/,
+	krb5_keyblock */*key*/);
+
 OM_uint32
 _gssapi_verify_mech_header (
 	u_char **/*str*/,
@@ -193,7 +205,7 @@ _gssapi_verify_mic_arcfour (
 	const gss_buffer_t /*token_buffer*/,
 	gss_qop_t * /*qop_state*/,
 	krb5_keyblock */*key*/,
-	char */*type*/);
+	const char */*type*/);
 
 OM_uint32
 _gssapi_verify_mic_cfx (
@@ -242,6 +254,28 @@ _gssapi_wrap_cfx_iov (
 	gss_iov_buffer_desc */*iov*/,
 	int /*iov_count*/);
 
+OM_uint32
+_gssapi_wrap_iov_arcfour (
+	OM_uint32 */*minor_status*/,
+	gsskrb5_ctx /*ctx*/,
+	krb5_context /*context*/,
+	int /*conf_req_flag*/,
+	int */*conf_state*/,
+	gss_iov_buffer_desc */*iov*/,
+	int /*iov_count*/,
+	krb5_keyblock */*key*/);
+
+OM_uint32
+_gssapi_wrap_iov_length_arcfour (
+	OM_uint32 */*minor_status*/,
+	gsskrb5_ctx /*ctx*/,
+	krb5_context /*context*/,
+	int /*conf_req_flag*/,
+	gss_qop_t /*qop_req*/,
+	int */*conf_state*/,
+	gss_iov_buffer_desc */*iov*/,
+	int /*iov_count*/);
+
 OM_uint32
 _gssapi_wrap_iov_length_cfx (
 	OM_uint32 */*minor_status*/,
@@ -278,7 +312,7 @@ OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_accept_sec_context (
 	OM_uint32 * /*minor_status*/,
 	gss_ctx_id_t * /*context_handle*/,
-	const gss_cred_id_t /*acceptor_cred_handle*/,
+	gss_const_cred_id_t /*acceptor_cred_handle*/,
 	const gss_buffer_t /*input_token_buffer*/,
 	const gss_channel_bindings_t /*input_chan_bindings*/,
 	gss_name_t * /*src_name*/,
@@ -291,7 +325,7 @@ _gsskrb5_accept_sec_context (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_acquire_cred (
 	OM_uint32 * /*minor_status*/,
-	const gss_name_t /*desired_name*/,
+	gss_const_name_t /*desired_name*/,
 	OM_uint32 /*time_req*/,
 	const gss_OID_set /*desired_mechs*/,
 	gss_cred_usage_t /*cred_usage*/,
@@ -299,11 +333,22 @@ _gsskrb5_acquire_cred (
 	gss_OID_set * /*actual_mechs*/,
 	OM_uint32 * time_rec );
 
+OM_uint32 GSSAPI_CALLCONV
+_gsskrb5_acquire_cred_ext (
+	OM_uint32 * /*minor_status*/,
+	gss_const_name_t /*desired_name*/,
+	gss_const_OID /*credential_type*/,
+	const void */*credential_data*/,
+	OM_uint32 /*time_req*/,
+	gss_const_OID /*desired_mech*/,
+	gss_cred_usage_t /*cred_usage*/,
+	gss_cred_id_t * output_cred_handle );
+
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_add_cred (
 	 OM_uint32 */*minor_status*/,
-	const gss_cred_id_t /*input_cred_handle*/,
-	const gss_name_t /*desired_name*/,
+	gss_const_cred_id_t /*input_cred_handle*/,
+	gss_const_name_t /*desired_name*/,
 	const gss_OID /*desired_mech*/,
 	gss_cred_usage_t /*cred_usage*/,
 	OM_uint32 /*initiator_time_req*/,
@@ -313,19 +358,24 @@ _gsskrb5_add_cred (
 	OM_uint32 */*initiator_time_rec*/,
 	OM_uint32 */*acceptor_time_rec*/);
 
+OM_uint32 GSSAPI_CALLCONV
+_gsskrb5_authorize_localname (
+	OM_uint32 */*minor_status*/,
+	gss_const_name_t /*input_name*/,
+	gss_const_buffer_t /*user_name*/,
+	gss_const_OID /*user_name_type*/);
+
 OM_uint32
 _gsskrb5_canon_name (
 	OM_uint32 */*minor_status*/,
 	krb5_context /*context*/,
-	int /*use_dns*/,
-	krb5_const_principal /*sourcename*/,
-	gss_name_t /*targetname*/,
+	gss_const_name_t /*targetname*/,
 	krb5_principal */*out*/);
 
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_canonicalize_name (
 	 OM_uint32 * /*minor_status*/,
-	const gss_name_t /*input_name*/,
+	gss_const_name_t /*input_name*/,
 	const gss_OID /*mech_type*/,
 	gss_name_t * output_name );
 
@@ -335,14 +385,14 @@ _gsskrb5_clear_status (void);
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_compare_name (
 	OM_uint32 * /*minor_status*/,
-	const gss_name_t /*name1*/,
-	const gss_name_t /*name2*/,
+	gss_const_name_t /*name1*/,
+	gss_const_name_t /*name2*/,
 	int * name_equal );
 
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_context_time (
 	OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	OM_uint32 * time_rec );
 
 OM_uint32
@@ -388,7 +438,7 @@ _gsskrb5_delete_sec_context (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_display_name (
 	OM_uint32 * /*minor_status*/,
-	const gss_name_t /*input_name*/,
+	gss_const_name_t /*input_name*/,
 	gss_buffer_t /*output_name_buffer*/,
 	gss_OID * output_name_type );
 
@@ -404,7 +454,7 @@ _gsskrb5_display_status (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_duplicate_name (
 	 OM_uint32 * /*minor_status*/,
-	const gss_name_t /*src_name*/,
+	gss_const_name_t /*src_name*/,
 	gss_name_t * dest_name );
 
 void
@@ -441,13 +491,13 @@ _gsskrb5_export_cred (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_export_name (
 	OM_uint32 * /*minor_status*/,
-	const gss_name_t /*input_name*/,
+	gss_const_name_t /*input_name*/,
 	gss_buffer_t exported_name );
 
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_export_sec_context (
-	 OM_uint32 * /*minor_status*/,
-	gss_ctx_id_t * /*context_handle*/,
+	 OM_uint32 */*minor_status*/,
+	gss_ctx_id_t */*context_handle*/,
 	gss_buffer_t interprocess_token );
 
 ssize_t
@@ -459,7 +509,7 @@ _gsskrb5_get_mech (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_get_mic (
 	OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	gss_qop_t /*qop_req*/,
 	const gss_buffer_t /*message_buffer*/,
 	gss_buffer_t message_token );
@@ -500,9 +550,9 @@ _gsskrb5_init (krb5_context */*context*/);
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_init_sec_context (
 	OM_uint32 * /*minor_status*/,
-	const gss_cred_id_t /*cred_handle*/,
+	gss_const_cred_id_t /*cred_handle*/,
 	gss_ctx_id_t * /*context_handle*/,
-	const gss_name_t /*target_name*/,
+	gss_const_name_t /*target_name*/,
 	const gss_OID /*mech_type*/,
 	OM_uint32 /*req_flags*/,
 	OM_uint32 /*time_req*/,
@@ -516,7 +566,7 @@ _gsskrb5_init_sec_context (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_context (
 	 OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	gss_name_t * /*src_name*/,
 	gss_name_t * /*targ_name*/,
 	OM_uint32 * /*lifetime_rec*/,
@@ -528,7 +578,7 @@ _gsskrb5_inquire_context (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_cred (
 	OM_uint32 * /*minor_status*/,
-	const gss_cred_id_t /*cred_handle*/,
+	gss_const_cred_id_t /*cred_handle*/,
 	gss_name_t * /*output_name*/,
 	OM_uint32 * /*lifetime*/,
 	gss_cred_usage_t * /*cred_usage*/,
@@ -537,7 +587,7 @@ _gsskrb5_inquire_cred (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_cred_by_mech (
 	 OM_uint32 * /*minor_status*/,
-	const gss_cred_id_t /*cred_handle*/,
+	gss_const_cred_id_t /*cred_handle*/,
 	const gss_OID /*mech_type*/,
 	gss_name_t * /*name*/,
 	OM_uint32 * /*initiator_lifetime*/,
@@ -547,14 +597,14 @@ _gsskrb5_inquire_cred_by_mech (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_cred_by_oid (
 	OM_uint32 * /*minor_status*/,
-	const gss_cred_id_t /*cred_handle*/,
+	gss_const_cred_id_t /*cred_handle*/,
 	const gss_OID /*desired_object*/,
 	gss_buffer_set_t */*data_set*/);
 
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_mechs_for_name (
 	 OM_uint32 * /*minor_status*/,
-	const gss_name_t /*input_name*/,
+	gss_const_name_t /*input_name*/,
 	gss_OID_set * mech_types );
 
 OM_uint32 GSSAPI_CALLCONV
@@ -566,7 +616,7 @@ _gsskrb5_inquire_names_for_mech (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_inquire_sec_context_by_oid (
 	OM_uint32 */*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	const gss_OID /*desired_object*/,
 	gss_buffer_set_t */*data_set*/);
 
@@ -588,9 +638,16 @@ OM_uint32
 _gsskrb5_lifetime_left (
 	OM_uint32 */*minor_status*/,
 	krb5_context /*context*/,
-	OM_uint32 /*lifetime*/,
+	OM_uint32 /*endtime*/,
 	OM_uint32 */*lifetime_rec*/);
 
+OM_uint32 GSSAPI_CALLCONV
+_gsskrb5_localname (
+	OM_uint32 */*minor_status*/,
+	gss_const_name_t /*pname*/,
+	const gss_OID /*mech_type*/,
+	gss_buffer_t /*localname*/);
+
 void *
 _gsskrb5_make_header (
 	void */*ptr*/,
@@ -601,7 +658,7 @@ _gsskrb5_make_header (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_process_context_token (
 	 OM_uint32 */*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	const gss_buffer_t token_buffer );
 
 OM_uint32 GSSAPI_CALLCONV
@@ -614,7 +671,9 @@ _gsskrb5_pseudo_random (
 	gss_buffer_t /*prf_out*/);
 
 OM_uint32
-_gsskrb5_register_acceptor_identity (const char */*identity*/);
+_gsskrb5_register_acceptor_identity (
+	OM_uint32 */*min_stat*/,
+	const char */*identity*/);
 
 OM_uint32
 _gsskrb5_release_buffer (
@@ -665,7 +724,7 @@ _gsskrb5_store_cred (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_unwrap (
 	OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	const gss_buffer_t /*input_message_buffer*/,
 	gss_buffer_t /*output_message_buffer*/,
 	int * /*conf_state*/,
@@ -689,7 +748,7 @@ _gsskrb5_verify_header (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_verify_mic (
 	OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	const gss_buffer_t /*message_buffer*/,
 	const gss_buffer_t /*token_buffer*/,
 	gss_qop_t * qop_state );
@@ -702,12 +761,12 @@ _gsskrb5_verify_mic_internal (
 	const gss_buffer_t /*message_buffer*/,
 	const gss_buffer_t /*token_buffer*/,
 	gss_qop_t * /*qop_state*/,
-	char * type );
+	const char * type );
 
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_wrap (
 	OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	int /*conf_req_flag*/,
 	gss_qop_t /*qop_req*/,
 	const gss_buffer_t /*input_message_buffer*/,
@@ -717,7 +776,7 @@ _gsskrb5_wrap (
 OM_uint32 GSSAPI_CALLCONV
 _gsskrb5_wrap_size_limit (
 	 OM_uint32 * /*minor_status*/,
-	const gss_ctx_id_t /*context_handle*/,
+	gss_const_ctx_id_t /*context_handle*/,
 	int /*conf_req_flag*/,
 	gss_qop_t /*qop_req*/,
 	OM_uint32 /*req_output_size*/,
